#include <tencentcloud/core/Credential.h>
#include <tencentcloud/core/profile/ClientProfile.h>
#include <tencentcloud/core/profile/HttpProfile.h>
#include <tencentcloud/tdmq/v20200217/TdmqClient.h>
#include <tencentcloud/tdmq/v20200217/model/CreateTopicRequest.h>
#include <tencentcloud/tdmq/v20200217/model/CreateTopicResponse.h>
#include <iostream>
#include <string>

using namespace TencentCloud;
using namespace TencentCloud::Tdmq::V20200217;
using namespace TencentCloud::Tdmq::V20200217::Model;
using namespace std;

/**
 * 创建topic示例
 */
int main() {
    /// 腾讯云账户SecretId
    /// 密钥可前往https://console.cloud.tencent.com/cam/capi网站进行获取
    string secretId = "AKIDSiiRtxxxx";
    /// 腾讯云账户SecretKey
    string secretKey = "GGzSeaM5xxxx";
    /// 云api地址
    string endpoint = "tdmq.tencentcloudapi.com";
    /// 集群所属地域参数：参考：https://cloud.tencent.com/document/api/1179/46067#.E5.9C.B0.E5.9F.9F.E5.88.97.E8.A1.A8
    string region = "ap-shanghai";
    /// 集群id
    string clusterId = "pulsar-xxx";
    /// 命名空间名称
    string environmentId = "cpp_http";
    /// topic名称
    string topicName = "topic1";
    /// 订阅名称
    string subName = "sub";
    /// 已授权角色密钥
    string token = "eyJrZXlJZC......";
    /// 已授权角色迷宫农村
    string userName = "admin";
    /// topic全称, 格式为集群（租户）ID/命名空间/Topic名称
    string fullTopicName = "pulsar-xxxx/cpp_http/topic1";

    // 认证信息
    Credential cred = Credential(secretId, secretKey);

    HttpProfile httpProfile = HttpProfile();
    httpProfile.SetEndpoint(endpoint);

    ClientProfile clientProfile = ClientProfile();
    clientProfile.SetHttpProfile(httpProfile);
    // 创建tdmq客户端
    TdmqClient client = TdmqClient(cred, region, clientProfile);

    /// 创建topic
    CreateTopicRequest topicRequest = CreateTopicRequest();
    topicRequest.SetClusterId(clusterId);
    topicRequest.SetEnvironmentId(environmentId);
    topicRequest.SetTopicName(topicName);
    topicRequest.SetPartitions(3);
    topicRequest.SetRemark("topic说明");
    topicRequest.SetTopicType(0);
    auto topicOutcome = client.CreateTopic(topicRequest);
    if (!topicOutcome.IsSuccess()) {
        cout << topicOutcome.GetError().PrintAll() << endl;
        return -1;
    }
    // 获取结果
    CreateTopicResponse topicResponse = topicOutcome.GetResult();
    cout << topicResponse.ToJsonString() << endl;

    /// 创建订阅
    CreateSubscriptionRequest subscriptionRequest = CreateSubscriptionRequest();
    subscriptionRequest.SetClusterId(clusterId);
    subscriptionRequest.SetEnvironmentId(environmentId);
    subscriptionRequest.SetTopicName(topicName);
    subscriptionRequest.SetSubscriptionName(subName);
    subscriptionRequest.SetRemark("订阅信息");
    subscriptionRequest.SetIsIdempotent(true);
    // 是否自动创建死信和重试主题，True 表示创建，False表示不创建，默认自动创建死信和重试主题。
    subscriptionRequest.SetAutoCreatePolicyTopic(false);
    // 指定死信和重试主题名称规范，LEGACY表示历史命名规则，COMMUNITY表示Pulsar社区命名规范
    subscriptionRequest.SetPostFixPattern("COMMUNITY");
    auto subOutcome = client.CreateSubscription(subscriptionRequest);
    if (!subOutcome.IsSuccess()) {
        cout << subOutcome.GetError().PrintAll() << endl;
        return -1;
    }
    // 获取结果
    CreateSubscriptionResponse subscriptionResponse = subOutcome.GetResult();
    cout << subscriptionResponse.ToJsonString() << endl;

    /// 发送消息
    SendMessagesRequest sendMessagesRequest = SendMessagesRequest();
    sendMessagesRequest.SetStringToken(token);
    sendMessagesRequest.SetProducerName(userName);
    // 设置topic名称, 格式为: 集群（租户）ID/命名空间/Topic名称
    sendMessagesRequest.SetTopic(fullTopicName);
    sendMessagesRequest.SetPayload("this is a new message.");
    sendMessagesRequest.SetSendTimeout(3000);
    auto sendOutcome = client.SendMessages(sendMessagesRequest);
    if (!sendOutcome.IsSuccess()) {
        cout << sendOutcome.GetError().PrintAll() << endl;
        return -1;
    }
    // 获取结果
    SendMessagesResponse sendMessagesResponse = sendOutcome.GetResult();
    cout << sendMessagesResponse.ToJsonString() << endl;

    /// 消费消息
    ReceiveMessageRequest receiveMessageRequest = ReceiveMessageRequest();
    receiveMessageRequest.SetTopic(fullTopicName);
    receiveMessageRequest.SetSubscriptionName(subName);
    receiveMessageRequest.SetReceiverQueueSize(10);
    receiveMessageRequest.SetSubInitialPosition("Latest");
    auto receiveOutcome = client.ReceiveMessage(receiveMessageRequest);
    if (!receiveOutcome.IsSuccess()) {
        cout << receiveOutcome.GetError().PrintAll() << endl;
        return -1;
    }
    // 获取结果
    ReceiveMessageResponse receiveMessageResponse = receiveOutcome.GetResult();
    cout << receiveMessageResponse.ToJsonString() << endl;

    /// 确认消息
    AcknowledgeMessageRequest acknowledgeMessageRequest = AcknowledgeMessageRequest();
    // 确认消息的id
    acknowledgeMessageRequest.SetMessageId(receiveMessageResponse.GetMessageID());
    // 设置topic名称, 格式为：集群（租户）ID/命名空间/Topic名称
    acknowledgeMessageRequest.SetAckTopic(fullTopicName);
    // 设置订阅信息
    acknowledgeMessageRequest.SetSubName(subName);
    // 确认消息
    auto ackOutcome = client.AcknowledgeMessage(acknowledgeMessageRequest);
    if (!ackOutcome.IsSuccess()) {
        cout << ackOutcome.GetError().PrintAll() << endl;
        return -1;
    }
    // 获取结果
    AcknowledgeMessageResponse acknowledgeMessageResponse = ackOutcome.GetResult();
    cout << acknowledgeMessageResponse.ToJsonString() << endl;
    return 0;
}